﻿using AirBomber.Entities;
using AirBomber.Rendering;
using System.Diagnostics.CodeAnalysis;

namespace AirBomber.Generics
{
    internal class BomberRendererEquatables : IEqualityComparer<BomberRendererBase?>
    {
        public bool Equals(BomberRendererBase? lhs, BomberRendererBase? rhs)
        {
            if (lhs is null || lhs.EntityBomber is null)
                throw new ArgumentNullException(nameof(lhs));

            if (rhs is null || rhs.EntityBomber is null)
                throw new ArgumentNullException(nameof(rhs));

            if (lhs.GetType().Name != rhs.GetType().Name)
                return false;

            if (lhs.EntityBomber.Speed != rhs.EntityBomber.Speed)
                return false;

            if (lhs.EntityBomber.Weight != rhs.EntityBomber.Weight)
                return false;

            if (lhs.EntityBomber.BodyColor != rhs.EntityBomber.BodyColor)
                return false;

            if (lhs is BomberRenderer && rhs is BomberRenderer)
            {
                BomberEntity lhsEntity = (BomberEntity)lhs.EntityBomber;
                BomberEntity rhsEntity = (BomberEntity)rhs.EntityBomber;

                if (lhsEntity.AdditionalColor != rhsEntity.AdditionalColor)
                    return false;

                if (lhsEntity.Bombs != rhsEntity.Bombs)
                    return false;

                if (lhsEntity.FuelTanks != rhsEntity.FuelTanks)
                    return false;
            }

            return true;
        }

        public int GetHashCode([DisallowNull] BomberRendererBase obj)
        {
            return obj.GetHashCode();
        }
    }
}
