﻿using AirBomber.Exceptions;

namespace AirBomber.Generics
{
    internal class SetGeneric<T>
        where T : class
    {
        private readonly List<T?> _objects;
        private readonly int _maxCount;

        public int Count => _objects.Count;

        public SetGeneric(int Count)
        {
            _maxCount = Count;
            _objects = new List<T?>(_maxCount);
        }

        public int Insert(T Entity, IEqualityComparer<T?>? EqualityComparer = null)
        {
            /** Вставка в начало набора */
            for (int i = 0; i < Count; i++)
                if (_objects[i] is null)
                {
                    if (EqualityComparer is not null && _objects.Contains(Entity, EqualityComparer))
                        throw new ArgumentException("Объект уже существует");

                    _objects[i] = Entity;
                    return i;
                }

            if (Count < _maxCount)
            {
                if (EqualityComparer is not null && _objects.Contains(Entity, EqualityComparer))
                    throw new ArgumentException("Объект уже существует");

                _objects.Add(Entity);
                return Count - 1;
            }

            throw new StorageOverflowException(_maxCount);
        }

        public int Insert(T Entity, int Position, IEqualityComparer<T?>? EqualityComparer = null)
        {
            if (Position >= _maxCount)
                throw new StorageOverflowException(_maxCount);

            if (_objects[Position] is null)
            {
                if (EqualityComparer is not null && _objects.Contains(Entity, EqualityComparer))
                    throw new ArgumentException("Объект уже существует");

                _objects[Position] = Entity;
                return Position;
            }

            throw new StorageOverflowException(_maxCount);
        }

        public bool Remove(int Position)
        {
            if (Position >= _maxCount)
                throw new EntityNotFoundException(Position);

            if (_objects[Position] is null)
                throw new EntityNotFoundException(Position);

            _objects[Position] = default(T);
            return true;
        }

        public T? this[int Position]
        {
            get
            {
                if (Position >= _maxCount)
                    return null;

                return _objects[Position];
            }
            set
            {
                if (Position >= _maxCount)
                    return;

                if (_objects[Position] is not null)
                    return;

                _objects[Position] = value;
            }
        }

        public IEnumerable<T?> GetEntities(int? MaxEntities = null)
        {
            for (int i = 0; i < _objects.Count; ++i)
            {
                yield return _objects[i];

                if (MaxEntities.HasValue && i == MaxEntities.Value)
                    yield break;
            }
        }

        public void SortSet(IComparer<T?> Comparer) => _objects.Sort(Comparer);
    }
}
